@@ -1,14 +1,11 @@
 #define SC(name) .nr_##name = SYS_##name
 
 {
-#ifdef SYS_mmap
-    SC(mmap),
-#else
-    .nr_mmap = -1,
-#endif
 #ifdef SYS_mmap2
+    .nr_mmap = -1,
     SC(mmap2),
 #else
+    SC(mmap),
     .nr_mmap2 = -1,
 #endif
     SC(munmap),
@@ -16,6 +13,7 @@
     SC(setsid),
     SC(setpgid),
     SC(fork),
+    .nr_clone = -1,
     SC(wait4),
 #ifdef SYS_signal
     SC(signal),
@@ -23,10 +21,11 @@
      .nr_signal = -1,
 #endif
     .nr_rt_sigaction = SYS_sigaction,
-    SC(open),
+    SC(openat),
     SC(close),
     SC(ioctl),
     SC(dup2),
+    .nr_dup3 = -1,
 #ifdef SYS_socketcall
     SC(socketcall),
 #else
